<?php

// 繰り返す回数
$limit = 5;

// 各プロセスを格納する
$processes = [];
for ($i = 0; $i < $limit; $i++) {
    $processes[] = $process = popen('php heavy_process.php', 'r');

    // ノンブロッキングモードにします。こうすることにより、全ての実行が終了するまで待機する必要がなくなります。
    stream_set_blocking($process, false);
}

// 読み込みがあるストリームを指定しますが、今回は使用しないため null です。
$read = null;

// 書き込み (echo や print) などが発生したストリームを指定するため、先程定義した $processes を渡します。
$write = $processes;

// 例外が発生したストリームを指定しますが、今回は使用しないため null です。
$except = null;

// 実行が終了したプロセスの数を数えます。
$proceeded = 0;


// 実行が終了したプロセスが繰り返す回数より少ない間繰り返します。
while ($proceeded < $limit) {
    // stream_select は指定した複数のストリームで何かしら処理が発生したものを、参照渡しの変数に変更を加えて使用できるようにします。
    // 今回の場合、書き込みが発生すると $write に変更があったストリーム、つまりプロセスの配列が格納されます。
    if (!stream_select($read, $write, $except, 200000)) {
        break;
    }

    foreach ($write as $writeStream) {
        // 書き込まれた値を出力します。
        echo stream_get_contents($writeStream);

        // プロセスが終了すると EOF になるため、後処理を行います。
        if (feof($writeStream)) {
            // 実行したプロセスのカウンタを一つ上げます。
            $proceeded++;

            // popen で開いたものを閉じてあげます。
            pclose($writeStream);

            // $write の中には既に終了したプロセスも混じっているため、そのプロセスを取り除いてあげます。
            $write = array_filter(
                $write,
                function ($stream) {
                    return is_resource($stream) && !feof($stream);
                }
            );
        }
    }
}
