<?php
$events = new \parallel\Events();

// (1) 読み込みイベントを発生させます。
$runtime1 = new \parallel\Runtime();
$future1 = $runtime1
    ->run(
        function () {
        }
    );

$events->addFuture('fire-read-event', $future1);

// (2) タスクのキャンセルイベントを発生させます。
$runtime2 = new \parallel\Runtime();
$future2 = $runtime2
    ->run(
        function () {
        }
    );

$future2->cancel();
$events->addFuture('fire-cancel-event', $future2);

// (3) タスクのクローズイベントを発生させます。
$runtime3 = new \parallel\Runtime();
$future3 = $runtime3
    ->run(
        function () {
        }
    );

// $runtime3 変数であることに注意してください。
$runtime3->close();
$events->addFuture('fire-close-event', $future3);

// (4) タスクの強制終了イベントを発生させます。
$runtime4 = new \parallel\Runtime();
$future4 = $runtime4
    ->run(
        function () {
        }
    );

// $runtime4 変数であることに注意してください。
$runtime4->kill();
$events->addFuture('fire-kill-event', $future4);

// (5) タスク内でのエラーイベントを発生させます
$runtime5 = new \parallel\Runtime();
$future5 = $runtime5
    ->run(
        function () {
            throw new RuntimeException();
        }
    );
$events->addFuture('fire-an-error-event', $future5);


// (6) タスク内で書き込みイベントを発生させます。
$channel = parallel\Channel::make('fire-write-event-on-channel', \parallel\Channel::Infinite);

// 書き込み用のコンテナを作成し events に追加します。
$input = new parallel\Events\Input();
$events->setInput($input);

// (7) チャネルへ書き込みを行います。
$input->add("fire-write-event-on-channel", "fired");

// チャネルを追加します。
$events->addChannel($channel);

/**
 * @var \parallel\Events\Event[] $events
 */
foreach ($events as $event) {
    switch ($event->type) {
        // 読み込みが発生した場合
        case \parallel\Events\Event\Type::Read:
            echo "{$event->source} で読み込みイベントが発生しました\n";
            break;

        // 書き込みが発生した場合
        case \parallel\Events\Event\Type::Write:
            echo "{$event->source} で書き込みイベントが発生しました\n";
            break;

        // タスクがクローズした場合
        case \parallel\Events\Event\Type::Close:
            echo "{$event->source} はクローズしました。\n";
            break;

        // タスクでキャンセルが発生した場合
        case \parallel\Events\Event\Type::Cancel:
            echo "{$event->source} はキャンセルされました\n";
            break;

        // タスクが強制終了させられた場合
        case \parallel\Events\Event\Type::Kill:
            echo "{$event->source} は強制終了されました\n";
            break;

        // タスク内でエラーが発生した場合
        case \parallel\Events\Event\Type::Error:
            echo "{$event->source} でエラーが発生しました\n";
            break;
    }
}
