<?php

$events = new parallel\Events();

// 時間がかかるタスクを登録していきます。
$tasks = [
    'A' => fn() => \parallel\run(
        function () {
            // 時間をかけるため sleep を使用します。
            // 本来は時間がかかりそうな処理をここに定義します。
            sleep(10);
        }
    ),
    'B' => fn() => \parallel\run(
        function () {
            // A と同様、時間をかけるため sleep を使用します。
            sleep(5);
        }
    ),
    'C' => fn() => \parallel\run(
        function () {
            // A と同様です。
            sleep(5);
        }
    ),
    'D' => fn() => \parallel\run(
        function () {
            // A と同様です。
            sleep(3);
        }
    ),
    'E' => fn() => \parallel\run(
        function () {
            // A と同様です。
            sleep(1);
        }
    ),
    'F' => fn() => \parallel\run(
        function () {
            // A と同様です。
            sleep(4);

            // バッチが失敗した場合を想定して、例外をスローします。
            throw new RuntimeException(
                '失敗しました。'
            );
        }
    ),
];

// Events に登録します。
foreach ($tasks as $name => $task) {
    $events->addFuture(
        $name,
        $task()
    );
}

// 時間を計測します。
$start = microtime(true);

/**
 * @var parallel\Events\Event[] $events
 */
foreach ($events as $event) {
    switch ($event->type) {
        case \parallel\Events\Event\Type::Read:
            echo "[{$event->source}] バッチが正常に完了しました。\n";
            break;
        case \parallel\Events\Event\Type::Error:
            echo "[{$event->source}] バッチの処理に失敗しました。\n";
            echo "[{$event->source}] エラーメッセージ: {$event->value->getMessage()}\n";
            break;
    }
}

$diff = ceil(microtime(true) - $start);

echo "トータル約 {$diff} 秒経過しました。\n";
