<?php

class Connection
{
    protected PDO $instance;

    protected string $host;
    protected string $user;
    protected string $pass;
    protected string $db;

    protected function __construct(string $host, string $user, string $pass, string $db)
    {
        $this->host = $host;
        $this->user = $user;
        $this->pass = $pass;
        $this->db = $db;
    }

    public static function factory()
    {
        static $instance;

        // 接続を使い回せるようにシングルトンで処理をします。
        // ユーザー名やパスワードは本来環境変数であったり、設定ファイルなどを用意するべきですが、今回は割愛して、
        // そのまま値を渡すようにしています。
        return $instance = $instance ?? new static(
            // データベースへ接続するためのホストを指定します。
                '127.0.0.1',

                // データベースへ接続するためのユーザー名を指定します。
                'root',

                // データベースへ接続するためのパスワードを指定します。
                // 今回 root のパスワードは空という指定をしているため、何も入力しないようにします。
                '',

                // 接続するデータベースを指定します。
                'php-asyncbook',
            );
    }

    public function getInstance(): PDO
    {
        $this->connectDatabaseIfNotConnected();
        return $this->instance;
    }

    public function connectDatabaseIfNotConnected(): void
    {
        $this->instance = new PDO(
            "mysql:dbname={$this->db};host={$this->host};charset=utf8mb4",
            $this->user,
            $this->pass
        );
    }
}