<?php

class LoginController
{
    public function index(SwooleServerRequest $serverRequest)
    {
        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/TemplateLogin.php', [
                'error_code' => (int)($serverRequest->getQueryParams()['error_code'] ?? 0),
            ])
        );
    }

    public function login(SwooleServerRequest  $serverRequest)
    {
        try {
            // POST されたデータを変数に格納します。
            $post = $serverRequest->getParsedBody();

            // ログイン名を取得します。
            $name = $post['name'] ?? null;

            // パスワードを取得します。
            $password = hash('SHA512', $post['password'] ?? '');


            // データベースを操作するための PDO を取得します。
            $pdo = Connection::factory()
                ->getInstance();

            $statement = $pdo->prepare(
                'SELECT * FROM `users` WHERE `name`=? AND `password`=? LIMIT 1'
            );

            $statement->bindValue(1, $name);
            $statement->bindValue(2, $password);

            $statement->execute();


            // ID を取得します。
            $result = $statement->fetch(PDO::FETCH_OBJ);

            if ($result) {
                // ログイン情報をセッションに格納します。
                $serverRequest
                    ->getSession()
                    ->write(
                        'user',
                        $result
                    );

                return (new SwooleResponse($serverRequest, ''))
                    // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                    ->withStatus(301)
                    ->withHeader(
                        'location',

                        // ログイン先はトップページとします。
                        "/"
                    );
            }

            throw new \League\Route\Http\Exception\BadRequestException(
                'ユーザーがいません',
                null,

                // エラーコードは 1 としておきます。
                1
            );

        } catch (\League\Route\Http\Exception\BadRequestException $e) {
            // 例外が発生した場合 location を設定して、登録ページに飛ばします。
            return (new SwooleResponse($serverRequest, ''))
                // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                ->withStatus(301)
                ->withHeader(
                    'location',
                    "/login?error_code={$e->getCode()}"
                );
        }
    }
}
