<?php

class ProfileController
{
    public function index(SwooleServerRequest $serverRequest, $params)
    {
        // 指定された ID を取得します。
        $id = $params['id'] ?? null;

        // ID が渡されていなければ
        if ($id === null) {
            return $this->error($serverRequest);
        }

        // データベースを操作するための PDO を取得します。
        $pdo = Connection::factory()
            ->getInstance();

        // 指定された ID でユーザーを検索します
        $statement = $pdo->prepare(
            'SELECT `name`, `avatar` FROM `users` WHERE `id`=? LIMIT 1'
        );

        // ID をバインドします
        $statement->bindValue(1, $id);
        $statement->execute();

        // ID を取得します。
        $result = $statement->fetch(PDO::FETCH_OBJ);

        // 存在しないアカウントの場合
        if (!$result) {
            return $this->error($serverRequest);
        }

        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/TemplateProfile.php', [
                'user' => $result,
            ])
        );
    }

    // よく使うのでメソッド化しています。
    protected function error(SwooleServerRequest $serverRequest)
    {
        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/Error.php', [
                'message' => 'ユーザーが存在しません。',
            ])
        );
    }
}
