<?php

class RegisterController
{
    public function index(SwooleServerRequest $serverRequest)
    {
        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/TemplateRegister.php', [
                'error_code' => (int) ($serverRequest->getQueryParams()['error_code'] ?? 0),
            ])
        );
    }

    public function success(SwooleServerRequest $serverRequest)
    {
        return new SwooleResponse(
            $serverRequest,
            loadTemplate('templates/TemplateRegisterSuccess.php')
        );
    }

    public function post(SwooleServerRequest $serverRequest)
    {
        try {
            // POST されたデータを変数に格納します。
            $post = $serverRequest->getParsedBody();

            // PHP の $_FILES に相当するものは下記のメソッドで取得可能です。
            $files = $serverRequest->getUploadedFiles();

            // 期待していない値が送られてきた場合
            if (!isset($post['name'], $post['password'], $files['avatar'])) {
                throw new \League\Route\Http\Exception\BadRequestException(
                    'パラメータが不正です。',
                    null,

                    // エラーコードは 1 としておきます。
                    1
                );
            }

            // それぞれ変数に格納していきます。
            $name = $post['name'];
            $password = $post['password'];

            // パスワードは 8 文字以上、かつ英数字のみとするようにします。
            if (strlen($password) < 8 || !ctype_alnum($password)) {
                throw new \League\Route\Http\Exception\BadRequestException(
                    'パスワードは 8 文字以上かつ、英数字のみ指定してください。',
                    null,

                    // エラーコードは 2 としておきます。
                    2
                );
            }

            // データベースを操作するための PDO を取得します。
            $pdo = Connection::factory()
                ->getInstance();

            // 既に登録されているユーザー名であるかを調べます。
            $statement = $pdo->prepare(
                'SELECT id FROM `users` WHERE `name`=? LIMIT 1'
            );

            // 値をバインドします。
            $statement->bindValue(1, $name);

            // プリペアドステートメントを実行します。
            $statement->execute();

            // ID を取得します。
            $result = $statement->fetch(PDO::FETCH_OBJ);

            // 既に登録済みのユーザー名だった場合
            if ($result) {
                throw new \League\Route\Http\Exception\BadRequestException(
                    '既に登録済みのユーザーです',
                    null,

                    // エラーコードは 3 としておきます。
                    3
                );
            }

            // アップロードされたファイルを読み込みます。
            $image = imagecreatefromstring(file_get_contents($files['avatar']['tmp_name']));

            // 画像を png に変換して変数に格納します。
            ob_start();
            imagepng($image);
            $avatar = ob_get_clean();

            // プリペアドステートメントを使用して、データベースに値を保存するようにします。
            $statement = $pdo->prepare(
                'INSERT INTO `users` (`name`, `password`, `avatar`, `created_at`, `updated_at`) VALUES (?, ?, ?, ?, ?)'
            );

            // 値をそれぞれバインドしていきます。
            $statement->bindValue(1, $name);
            $statement->bindValue(2, hash('SHA512', $password));

            // データベースに保存するときに base64 にして保存するようにします。
            $statement->bindValue(3, base64_encode($avatar));
            $statement->bindValue(4, date('Y/m/d H:i:s'));
            $statement->bindValue(5, date('Y/m/d H:i:s'));

            $statement->execute();

            // 保存に成功した場合 success ページへリダイレクトするようにします。
            return (new SwooleResponse($serverRequest, ''))
                // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                ->withStatus(301)
                ->withHeader(
                    'location',
                    "/register/success"
                );
        } catch (\League\Route\Http\Exception\BadRequestException $e) {
            // 例外が発生した場合 location を設定して、登録ページに飛ばします。
            return (new SwooleResponse($serverRequest, ''))
                // リダイレクトはステータスコードを 301 に指定する必要があるので、設定します。
                ->withStatus(301)
                ->withHeader(
                    'location',
                    "/register?error_code={$e->getCode()}"
                );
        }
    }
}
