<?php

class Session
{
    protected $request;
    protected array $data = [];
    protected string $sessionName;
    protected ?string $sessionId;

    public function __construct(SwooleServerRequest $request)
    {
        $this->request = $request;

        // セッション名を取得します。
        $this->sessionName = session_name();

        // Cookie からセッションを取得します。
        $this->sessionId = $this->request->getCookieParams()[$this->sessionName]
            ?? null;

        // Cookie が定義されていない場合 Cookie に保存します。
        // またはセッションファイルが存在しない場合も Cookie に保存するようにします。
        if (!$this->sessionId || !is_file($this->getSessionFilePath())) {
            // 適当な時間と適当な文字列でセッション ID を作成します。
            $this->sessionId = hash('SHA512', 'PHP_ASYNC_BOOK_' . microtime(true));

            // Cookie に書き込みます。
            $this->request
                ->getResponse()
                ->cookie(
                    $this->sessionName,
                    $this->sessionId,

                );
        } else {
            // ファイルが存在していれば、データを読み込むようにします。
            // なお、同時読み込みが発生する可能性があるため、file_get_contents ではなく fopen を使用します。
            $handle = fopen($this->getSessionFilePath(), 'r');
            flock($handle, LOCK_EX);
            $this->data = unserialize(
                stream_get_contents($handle)
            );
            flock($handle, LOCK_UN);
        }
    }

    public function write(string $key, $value): self
    {
        $this->data[$key] = $value;
        return $this;
    }

    public function read(string $key)
    {
        return $this->data[$key] ?? null;
    }

    public function __destruct()
    {
        // セッションディレクトリがなければ作成ししておきます。
        @mkdir(__DIR__ . '/session', 0777, true);

        // 作成したディレクトリにセッションファイルを保管するようにします。
        file_put_contents(
            $this->getSessionFilePath(),
            serialize($this->data),
            LOCK_EX
        );
    }

    public function getSessionFilePath(): string
    {
        return __DIR__ . '/session/' . $this->sessionId;
    }
}