<?php

class SwooleBodyStream implements \Psr\Http\Message\StreamInterface
{
    protected $body;
    protected string $rawBody;
    protected int $size;

    public function __construct(string $body)
    {
        $this->size = strlen($body);
        $this->rawBody = $body;
        $this->body = fopen('php://memory', 'r+');
        fwrite($this->body, $body);
        $this->rewind();
    }

    /**
     * 渡されたストリームを，文字列として取得し，そのまま帰すためのメソッドです。
     */
    public function __toString()
    {
        return $this->getContents();
    }

    /**
     * オープンされたストリームを閉じるためのメソッドです。
     */
    public function close()
    {
        fclose($this->body);
    }

    /**
     * オープンしたストリームを削除するためのメソッドです。
     */
    public function detach()
    {
        unset($this->body);
        return $this->body;
    }

    /**
     * 文字列の長さを返すためのメソッドです。
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * 文字列をどこまで取得しているかを返すためのメソッド尾です。
     */
    public function tell()
    {
        return ftell($this->body);
    }

    /**
     * 文字列を最後まで読み終わっているかどうかを取得するためのメソッドです。
     */
    public function eof()
    {
        return feof($this->body);
    }

    /**
     * 読み込む位置を変更できるかどうかを返すためのメソッドです。
     * 本書の例では変更できるため，常に true を返すようにしています。
     */
    public function isSeekable()
    {
        return true;
    }

    /**
     * 読み込みたい文字列の位置を変更するためのメソッドです。
     */
    public function seek($offset, $whence = SEEK_SET)
    {
        fseek($this->body, $whence);
    }

    /**
     * 読み込みたい文字列の位置を 0 にするためのメソッドです。
     */
    public function rewind()
    {
        rewind($this->body);
    }

    /**
     * ストリームに新しく書き込めるかどうかを返すためのメソッドです。
     */
    public function isWritable()
    {
        return true;
    }

    /**
     * ストリームに新しく文字列を書き込むためのメソッドです。
     */
    public function write($string)
    {
        fwrite($this->body, $string);
    }

    /**
     * ストリームから読み込めるかどうかを取得するためのメソッドです。
     */
    public function isReadable()
    {
        return true;
    }

    /**
     * ストリームから文字列を読み込むためのメソッドです。
     */
    public function read($length)
    {
        return fread($this->body, $length);
    }

    /**
     * もともとの文字列を返すためのメソッドです。
     */
    public function getContents()
    {
        return $this->rawBody;
    }

    /**
     * ストリームのメタ情報（どういった種類のストリームなのか，など）を取得する貯めのメソッドです。
     */
    public function getMetadata($key = null)
    {
        $meta = stream_get_meta_data($this->body);
        return $meta[$key] ?? $meta;
    }
}
