<?php


class SwooleUri implements \Psr\Http\Message\UriInterface
{
    protected $uri;

    public function __construct(string $uri)
    {
        $this->uri = parse_url($uri);
    }

    public function getScheme()
    {
        return $this->uri['scheme'] ?? '';
    }

    public function getAuthority()
    {
        return ($this->uri['user'] ?? '') . (($this->uri['pass'] ?? null) !== null ? ":{$this->uri['pass']}" : '');
    }

    public function getUserInfo()
    {
        return ($this->getAuthority() ? "{$this->getAuthority()}@" : '') . $this->getHost() . ($this->getPort() !== 80 ? ":{$this->getPort()}" : '');
    }

    public function getHost()
    {
        return $this->uri['host'] ?? '/';
    }

    public function getPort()
    {
        return (int) ($this->uri['port'] ?? 80);
    }

    public function getPath()
    {
        return $this->uri['path'] ?? '/';
    }

    public function getQuery()
    {
        return $this->uri['query'] ?? '';
    }

    public function getFragment()
    {
        return $this->uri['fragment'] ?? '';
    }

    public function withScheme($scheme)
    {
        $this->uri['scheme'] = $scheme;
    }

    public function withUserInfo($user, $password = null)
    {
        $this->uri['user'] = $user;
        if ($password) {
            $this->uri['pass'] = $password;
        }
    }

    public function withHost($host)
    {
        $this->uri['host'] = $host;
    }

    public function withPort($port)
    {
        $this->uri['port'] = (int) $port;
    }

    public function withPath($path)
    {
        $this->uri['path'] = $path;
    }

    public function withQuery($query)
    {
        $this->uri['query'] = $query;
    }

    public function withFragment($fragment)
    {
        $this->uri['fragment'] = $fragment;
    }

    public function __toString()
    {
        return $this->getScheme()
            . '://'
            . $this->getUserInfo()
            . $this->getPath()
            . ($this->getQuery() ? '?' . $this->getQuery() : '')
            . ($this->getFragment() ? '#' . $this->getFragment() : '');
    }
}