#!/usr/bin/env bash

# 終了シグナルが送られたタイミングで終了させるようにします。
trap "trap - SIGTERM && kill -- -$$" SIGINT SIGTERM EXIT

# 複数回呼ぶため関数にしておきます
run() {
    # server.php をバックグラウンドで実行させます
    php server.php &
    pid=$!
}

# 初回起動時に動作させる
run

# fswatch を用いてファイルの変更を検知します。今回検知するイベントは更新のみの Updated です。
# また server.sh と vendor 自身は検知しないように除外しておきます。
fswatch --exclude="$0" --exclude "vendor/" --one-per-batch --event=Updated . | while read line; do
    # プロセスを終了させます。
    if [[ -n $pid ]]; then
        kill $pid
    fi

    # 終了した後にもう一度走らせます
    run
done